@@ -29,26 +29,21 @@
 #define MKNOB_DEFAULTH 100
 #define MKNOB_DEFAULTSIZE 25
 #define MKNOB_MINSIZE 12
-#define MKNOB_THICK 3
 /* ------------ mknob  ----------------------- */
 typedef struct _mknob
 {
     t_iemgui x_gui;
-    int      x_prev_h;
-    int      x_prev_w;
     int      x_pos;
     int      x_val;
-    int      x_prev_val;
-    int      x_center;
-    int      x_thick;
     int      x_lin0_log1;
     int      x_steady;
     double   x_min;
     double   x_max;
     int   	 x_H;
-    int   	 x_prev_H;
     double   x_k;
-    int      x_reverse;
+    t_float  x_fval;
+    int		 x_show_io;
+    int 	 x_io_visible;
 } t_mknob;
 
 t_widgetbehavior mknob_widgetbehavior;
@@ -57,33 +52,26 @@ static t_class *mknob_class;
 static int compat = 0; // current running pd version < 0.47
 
 /* widget helper functions */
+
+static void mknob_draw_io(t_mknob *x,t_glist *glist);
+
 static void mknob_update_knob(t_mknob *x, t_glist *glist)
 {
+    t_canvas *canvas = glist_getcanvas(glist);
+    float val = (x->x_val + 50.0) / 100.0 / x->x_H;
+    float angle;
+    float radius = x->x_gui.x_w / 2.0;
+    float miniradius = radius / 6.0;
+    int x0, y0, x1, y1, xc, yc, xp, yp, xpc, ypc;
 
-    /* only draw if something changed */
-    /*if(!(x->x_val != x->x_prev_val || x->x_gui.x_h != x->x_prev_h ||
-       x->x_gui.x_w != x->x_prev_w || x->x_H != x->x_prev_H))
-        return;*/ /* actually seems not needed since value changed is already done ;
-	             and moving the knob was bugged because of that test.
-	          */
+    if(miniradius < 3.0) miniradius = 3.0;
 
-    x->x_prev_val = x->x_val;
-    x->x_prev_h = x->x_gui.x_h;
-    x->x_prev_w = x->x_gui.x_w;
-    x->x_prev_H = x->x_H;
-    t_canvas *canvas=glist_getcanvas(glist);
-//	float val=(x->x_val + 50.0)/100.0/MKNOB_TANGLE;
-    float val=(x->x_val + 50.0)/100.0/x->x_H;
-    float 	angle,
-            radius=x->x_gui.x_w/2.0,
-            miniradius=MKNOB_THICK;
-    int x0,y0,x1,y1,xc,yc,xp,yp,xpc,ypc;
-    x0=text_xpix(&x->x_gui.x_obj, glist);
-    y0=text_ypix(&x->x_gui.x_obj, glist);
-    x1=x0+x->x_gui.x_w;
-    y1=y0+x->x_gui.x_w;
-    xc=(x0+x1)/2;
-    yc=(y0+y1)/2;
+    x0 = text_xpix(&x->x_gui.x_obj, glist);
+    y0 = text_ypix(&x->x_gui.x_obj, glist);
+    x1 = x0 + x->x_gui.x_w;
+    y1 = y0 + x->x_gui.x_w;
+    xc = (x0 + x1) / 2;
+    yc = (y0 + y1) / 2;
 
     if(x->x_gui.x_h<0)
         angle=val*(M_PI*2)+M_PI/2.0;
@@ -95,8 +83,13 @@ static void mknob_update_knob(t_mknob *x, t_glist *glist)
     xpc=miniradius*cos(angle-M_PI/2);
     ypc=miniradius*sin(angle-M_PI/2);
 
-    sys_vgui(".x%lx.c coords %xKNOB %d %d %d %d %d %d\n",
+    sys_vgui(".x%lx.c coords %lxKNOB %d %d %d %d %d %d\n",
              canvas,x,xp,yp,xc+xpc,yc+ypc,xc-xpc,yc-ypc);
+
+	if(x->x_show_io && !x->x_gui.x_fsf.x_selected) {
+		x->x_show_io = 0;
+		mknob_draw_io(x, glist);
+	}
 }
 
 static void mknob_draw_update(t_mknob *x, t_glist *glist)
@@ -104,23 +97,6 @@ static void mknob_draw_update(t_mknob *x, t_glist *glist)
     if (glist_isvisible(glist))
     {
         mknob_update_knob(x,glist);
-
-        /*if(x->x_val == x->x_center)
-        {
-            if(!x->x_thick)
-            {
-        	sys_vgui(".x%lx.c itemconfigure %xKNOB -width 7\n", canvas, x);
-        	x->x_thick = 1;
-            }
-        }
-        else
-        {
-            if(x->x_thick)
-            {
-        	sys_vgui(".x%lx.c itemconfigure %xKNOB -width 3\n", canvas, x);
-        	x->x_thick = 0;
-            }
-        }*/
     }
 }
 
@@ -134,42 +110,38 @@ static void mknob_draw_new(t_mknob *x, t_glist *glist)
     t_canvas *canvas=glist_getcanvas(glist);
 
     if(compat) {
-        sys_vgui(".x%lx.c create oval %d %d %d %d -fill #%6.6x -tags %xBASE\n",
+        sys_vgui(".x%lx.c create oval %d %d %d %d -fill #%6.6x -tags %lxBASE\n",
                  canvas,xpos,ypos,xpos + x->x_gui.x_w, ypos + x->x_gui.x_w,
                  x->x_gui.x_bcol,x);
-        sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d -fill #%6.6x -tags %xKNOB\n",
+        sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d -fill #%6.6x -tags %lxKNOB\n",
                  glist_getcanvas(glist),
                  xc,ypos,xc-4,yc,xc+4,yc,x->x_gui.x_fcol,x);
         mknob_update_knob(x,glist);
         sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-	         -font {{%s} -%d %s} -fill #%6.6x -tags %xLABEL\n",
+	         -font {{%s} -%d %s} -fill #%6.6x -tags %lxLABEL\n",
                  canvas, xpos+x->x_gui.x_ldx,
                  ypos+x->x_gui.x_ldy,
                  strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
                  x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight, x->x_gui.x_lcol, x);
     } else {
-        sys_vgui(".x%lx.c create oval %d %d %d %d -fill #%06x -tags %xBASE\n",
+        sys_vgui(".x%lx.c create oval %d %d %d %d -fill #%06x -width %d -tags %lxBASE\n",
                  canvas,xpos,ypos,xpos + x->x_gui.x_w, ypos + x->x_gui.x_w,
-                 x->x_gui.x_bcol,x);
-        sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d -fill #%06x -tags %xKNOB\n",
+                 x->x_gui.x_bcol, IEMGUI_ZOOM(x),
+                 x);
+        sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d -fill #%06x -tags %lxKNOB\n",
                  glist_getcanvas(glist),
                  xc,ypos,xc-4,yc,xc+4,yc,x->x_gui.x_fcol,x);
         mknob_update_knob(x,glist);
         sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-	     -font {{%s} -%d %s} -fill #%06x -tags %xLABEL\n",
-             canvas, xpos+x->x_gui.x_ldx,
-             ypos+x->x_gui.x_ldy,
+	     -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
+             canvas, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x),
              strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
-             x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight, x->x_gui.x_lcol, x);
+             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight, 
+             x->x_gui.x_lcol, x);
     }
-    /*if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %xOUT%d\n",
-         canvas, xpos+ x->x_gui.x_w/2-3, ypos + x->x_gui.x_w-1,
-         xpos+ x->x_gui.x_w/2+4, ypos + x->x_gui.x_w, x, 0);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-    sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %xIN%d\n",
-         canvas, xpos+ x->x_gui.x_w/2-3, ypos,
-         xpos+ x->x_gui.x_w/2+4, ypos+1, x, 0);*/
+    
+    x->x_io_visible = 0;
 }
 
 static void mknob_draw_move(t_mknob *x, t_glist *glist)
@@ -177,38 +149,46 @@ static void mknob_draw_move(t_mknob *x, t_glist *glist)
     int xpos=text_xpix(&x->x_gui.x_obj, glist);
     int ypos=text_ypix(&x->x_gui.x_obj, glist);
     int r = xpos + (x->x_val + 50)/100;
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
     t_canvas *canvas=glist_getcanvas(glist);
 
-    sys_vgui(".x%lx.c coords %xBASE %d %d %d %d\n",
+    sys_vgui(".x%lx.c coords %lxBASE %d %d %d %d\n",
              canvas, x,
              xpos, ypos,
              xpos + x->x_gui.x_w, ypos + x->x_gui.x_w);
+    if(x->x_io_visible) {
+    	sys_vgui(".x%lx.c coords %lxBORDER %d %d %d %d\n",
+             canvas, x,
+             xpos, ypos,
+             xpos + x->x_gui.x_w, ypos + x->x_gui.x_w);
+        sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
+             canvas, x, 0,
+             xpos, ypos + x->x_gui.x_w + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_w);
+        sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
+             canvas, x, 0,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh);
+	}
     mknob_update_knob(x,glist);
-    sys_vgui(".x%lx.c coords %xLABEL %d %d\n",
-             canvas, x, xpos+x->x_gui.x_ldx, ypos+x->x_gui.x_ldy);
-    /*if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c coords %xOUT%d %d %d %d %d\n",
-         canvas, x, 0,
-         xpos+ x->x_gui.x_w/2-3, ypos + x->x_gui.x_w-1,
-         xpos+ x->x_gui.x_w/2+4, ypos + x->x_gui.x_w);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-    sys_vgui(".x%lx.c coords %xIN%d %d %d %d %d\n",
-         canvas, x, 0,
-         xpos+ x->x_gui.x_w/2-3, ypos,
-         xpos+ x->x_gui.x_w/2+4, ypos+1);*/
+    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
+             canvas, x, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x));
 }
 
 static void mknob_draw_erase(t_mknob *x,t_glist *glist)
 {
     t_canvas *canvas=glist_getcanvas(glist);
 
-    sys_vgui(".x%lx.c delete %xBASE\n", canvas, x);
-    sys_vgui(".x%lx.c delete %xKNOB\n", canvas, x);
-    sys_vgui(".x%lx.c delete %xLABEL\n", canvas, x);
-    /*if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c delete %xOUT%d\n", canvas, x, 0);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c delete %xIN%d\n", canvas, x, 0);*/
+    sys_vgui(".x%lx.c delete %lxBASE\n", canvas, x);
+    sys_vgui(".x%lx.c delete %lxKNOB\n", canvas, x);
+    sys_vgui(".x%lx.c delete %lxLABEL\n", canvas, x);
+    if(x->x_io_visible) {
+        sys_vgui(".x%lx.c delete %lxBORDER\n", canvas, x);
+        sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
+        sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
+    	x->x_io_visible = 0;
+    }
 }
 
 static void mknob_draw_config(t_mknob *x,t_glist *glist)
@@ -216,41 +196,54 @@ static void mknob_draw_config(t_mknob *x,t_glist *glist)
     t_canvas *canvas=glist_getcanvas(glist);
 
     if(compat) {
-        sys_vgui(".x%lx.c itemconfigure %xLABEL -font {{%s} -%d %s} -fill #%6.6x -text {%s} \n",
-                 canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
+        sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%6.6x -text {%s} \n",
+                 canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
                  x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
                  strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
-        sys_vgui(".x%lx.c itemconfigure %xKNOB -fill #%6.6x\n", canvas, x, x->x_gui.x_fcol);
-        sys_vgui(".x%lx.c itemconfigure %xBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
+        sys_vgui(".x%lx.c itemconfigure %lxKNOB -fill #%6.6x -width %d\n", canvas, x, x->x_gui.x_fcol, IEMGUI_ZOOM(x));
+        sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
     } else {
-        sys_vgui(".x%lx.c itemconfigure %xLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
-                 canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
+        sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
+                 canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
                  x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
                  strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
-        sys_vgui(".x%lx.c itemconfigure %xKNOB -fill #%06x\n", canvas, x, x->x_gui.x_fcol);
-        sys_vgui(".x%lx.c itemconfigure %xBASE -fill #%06x\n", canvas, x, x->x_gui.x_bcol);
+        sys_vgui(".x%lx.c itemconfigure %lxKNOB -fill #%06x -width %d\n", canvas, x, x->x_gui.x_fcol, IEMGUI_ZOOM(x));
+        sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%06x\n", canvas, x, x->x_gui.x_bcol);
     }
 }
 
-/*static void mknob_draw_io(t_mknob *x,t_glist *glist, int old_snd_rcv_flags)
+static void mknob_draw_io(t_mknob *x,t_glist *glist)
 {
     int xpos=text_xpix(&x->x_gui.x_obj, glist);
     int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    t_canvas *canvas=glist_getcanvas(glist);*/
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas=glist_getcanvas(glist);
 
-    /*if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %xOUT%d\n",
-         canvas, xpos+ x->x_gui.x_w/2-3, ypos + x->x_gui.x_w-1,
-         xpos+ x->x_gui.x_w/2+4, ypos + x->x_gui.x_w, x, 0);
-    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c delete %xOUT%d\n", canvas, x, 0);
-    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %xIN%d\n",
-         canvas, xpos+ x->x_gui.x_w/2-3, ypos,
-         xpos+ x->x_gui.x_w/2+4, ypos+1, x, 0);
-    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
-    sys_vgui(".x%lx.c delete %xIN%d\n", canvas, x, 0);*/
-//}
+    if(x->x_show_io && !x->x_io_visible) {
+	    sys_vgui(".x%lx.c create rectangle %d %d %d %d -width %d -tags %lxBORDER\n",
+             canvas, xpos, ypos,
+             xpos + x->x_gui.x_w, ypos + x->x_gui.x_w,
+             IEMGUI_ZOOM(x),
+             x);
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
+             canvas,
+             xpos, ypos + x->x_gui.x_w + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_w,
+             x, 0);
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
+             canvas,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
+        x->x_io_visible = 1;
+    }
+    else if((!x->x_show_io) && x->x_io_visible) {
+        sys_vgui(".x%lx.c delete %lxBORDER\n", canvas, x);
+        sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
+    	sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
+        x->x_io_visible = 0;
+	}
+}
 
 static void mknob_draw_select(t_mknob *x,t_glist *glist)
 {
@@ -259,30 +252,37 @@ static void mknob_draw_select(t_mknob *x,t_glist *glist)
     if(compat) {
         if(x->x_gui.x_fsf.x_selected)
         {
-            //pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
-            sys_vgui(".x%lx.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
-            sys_vgui(".x%lx.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
+            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
+            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
         }
         else
         {
-            //pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
-            sys_vgui(".x%lx.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
-            sys_vgui(".x%lx.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
+            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
+            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
         }
     } else  {
         if(x->x_gui.x_fsf.x_selected)
         {
-            //pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
-            sys_vgui(".x%lx.c itemconfigure %xBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
-            sys_vgui(".x%lx.c itemconfigure %xLABEL -fill #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
+            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
+            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
         }
         else
         {
-            //pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
-            sys_vgui(".x%lx.c itemconfigure %xBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
-            sys_vgui(".x%lx.c itemconfigure %xLABEL -fill #%06x\n", canvas, x, x->x_gui.x_lcol);
+            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
+            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n", canvas, x, x->x_gui.x_lcol);
         }
     }
+    
+    if(x->x_gui.x_fsf.x_selected) {
+    	x->x_show_io = 1;
+    	mknob_draw_io(x, glist);
+    }
+    if(x->x_io_visible){
+    	if(x->x_gui.x_fsf.x_selected)
+    		sys_vgui(".x%lx.c itemconfigure %lxBORDER -outline #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);		
+		else
+			sys_vgui(".x%lx.c itemconfigure %lxBORDER -outline #%06x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
+	}
 }
 
 void mknob_draw(t_mknob *x, t_glist *glist, int mode)
@@ -305,16 +305,16 @@ void mknob_draw(t_mknob *x, t_glist *glist, int mode)
 
 /* ------------------------ mknob widgetbehaviour----------------------------- */
 
-
+#define GRECTRATIO 0
 static void mknob_getrect(t_gobj *z, t_glist *glist,
                           int *xp1, int *yp1, int *xp2, int *yp2)
 {
     t_mknob *x = (t_mknob *)z;
 
-    *xp1 = text_xpix(&x->x_gui.x_obj, glist);
-    *yp1 = text_ypix(&x->x_gui.x_obj, glist);
-    *xp2 = *xp1 + x->x_gui.x_w;
-    *yp2 = *yp1 + x->x_gui.x_w;
+    *xp1 = text_xpix(&x->x_gui.x_obj, glist) + GRECTRATIO * x->x_gui.x_w;
+    *yp1 = text_ypix(&x->x_gui.x_obj, glist) + GRECTRATIO * x->x_gui.x_w;
+    *xp2 = text_xpix(&x->x_gui.x_obj, glist) + x->x_gui.x_w * (1 - GRECTRATIO);
+    *yp2 = text_ypix(&x->x_gui.x_obj, glist) + x->x_gui.x_w * (1 - GRECTRATIO);
 }
 
 static void mknob_save(t_gobj *z, t_binbuf *b)
@@ -329,7 +329,7 @@ static void mknob_save(t_gobj *z, t_binbuf *b)
         binbuf_addv(b, "ssiisiiffiisssiiiiiiiii", gensym("#X"),gensym("obj"),
             (t_int)x->x_gui.x_obj.te_xpix, (t_int)x->x_gui.x_obj.te_ypix,
             atom_getsymbol(binbuf_getvec(x->x_gui.x_obj.te_binbuf)),
-            x->x_gui.x_w, x->x_gui.x_h,
+            x->x_gui.x_w/IEMGUI_ZOOM(x), x->x_gui.x_h/IEMGUI_ZOOM(x),
             (float)x->x_min, (float)x->x_max,
             x->x_lin0_log1, iem_symargstoint(&x->x_gui.x_isa),
             srl[0], srl[1], srl[2],
@@ -356,37 +356,33 @@ static void mknob_save(t_gobj *z, t_binbuf *b)
     binbuf_addv(b, ";");
 }
 
+void mknob_update_H(t_mknob *x)
+{
+    int H;
+
+    H = x->x_gui.x_h;
+    if(H < 0) H = 360;
+    if(H == 0) H = 270;
+    x->x_H = H;
+
+    if(x->x_lin0_log1)
+        x->x_k = log(x->x_max/x->x_min)/(double)(x->x_H/IEMGUI_ZOOM(x) - 1);
+    else
+        x->x_k = (x->x_max - x->x_min)/(double)(x->x_H/IEMGUI_ZOOM(x) - 1);
+}
+
 void mknob_check_wh(t_mknob *x, int w, int h)
 {
     int H;
 
     if(w < MKNOB_MINSIZE) w = MKNOB_MINSIZE;
-    x->x_gui.x_w = w;
+    x->x_gui.x_w = w * IEMGUI_ZOOM(x);
 
-    if(h < -1) h=-1;
-    if((h>0)&&(h<20)) h=20;
-    x->x_gui.x_h = h;
+    if(h < -1) h = -1;
+    if((h > 0) && (h < 20)) h = 20;
+    x->x_gui.x_h = h * IEMGUI_ZOOM(x);
 
-    H=x->x_gui.x_h;
-    if(H<0) H=360;
-    if(H==0) H=270;
-    x->x_H=H;
-
-    if(x->x_lin0_log1)
-        x->x_k = log(x->x_max/x->x_min)/(double)(x->x_H - 1);
-    else
-        x->x_k = (x->x_max - x->x_min)/(double)(x->x_H - 1);
-
-    /*x->x_center = (x->x_gui.x_w-1)*50;
-    if(x->x_val > (x->x_gui.x_w*100 - 100))
-    {
-    x->x_pos = x->x_gui.x_w*100 - 100;
-    x->x_val = x->x_pos;
-    }
-    if(x->x_lin0_log1)
-    x->x_k = log(x->x_max/x->x_min)/(double)(x->x_gui.x_w - 1);
-    else
-    x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);*/
+    mknob_update_H(x);
 }
 
 void mknob_check_minmax(t_mknob *x, double min, double max)
@@ -410,19 +406,11 @@ void mknob_check_minmax(t_mknob *x, double min, double max)
     }
     x->x_min = min;
     x->x_max = max;
-    if(x->x_min > x->x_max)                /* bugfix */
-        x->x_reverse = 1;
-    else
-        x->x_reverse = 0;
 
     if(x->x_lin0_log1)
-        x->x_k = log(x->x_max/x->x_min)/(double)(x->x_H - 1);
+        x->x_k = log(x->x_max/x->x_min)/(double)(x->x_H/IEMGUI_ZOOM(x) - 1);
     else
-        x->x_k = (x->x_max - x->x_min)/(double)(x->x_H - 1);
-    /*if(x->x_lin0_log1)
-    	x->x_k = log(x->x_max/x->x_min)/(double)(MKNOB_TANGLE - 1);
-    else
-    	x->x_k = (x->x_max - x->x_min)/(double)(MKNOB_TANGLE - 1);*/
+        x->x_k = (x->x_max - x->x_min)/(double)(x->x_H/IEMGUI_ZOOM(x) - 1);
 }
 
 static void mknob_properties(t_gobj *z, t_glist *owner)
@@ -433,14 +421,14 @@ static void mknob_properties(t_gobj *z, t_glist *owner)
 
     iemgui_properties(&x->x_gui, srl);
     if(compat) sprintf(buf, "pdtk_iemgui_dialog %%s mknob \
-	    --------dimension(pix):-------- %d %d width: %d %d mouse: \
+	    --------dimension(pix):-------- %d %d size: %d %d mouse: \
 	    -----------output-range:----------- %g left: %g right: %g \
 	    %d lin log %d %d empty %d \
 	    %s %s \
 	    %s %d %d \
 	    %d %d \
 	    %d %d %d\n",
-            x->x_gui.x_w, MKNOB_MINSIZE, x->x_gui.x_h, -1,
+            x->x_gui.x_w/IEMGUI_ZOOM(x), MKNOB_MINSIZE, x->x_gui.x_h/IEMGUI_ZOOM(x), -1,
             x->x_min, x->x_max, 0.0,/*no_schedule*/
             x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, x->x_steady, -1,/*no multi, but iem-characteristic*/
             srl[0]->s_name, srl[1]->s_name,
@@ -448,14 +436,14 @@ static void mknob_properties(t_gobj *z, t_glist *owner)
             x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
             0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol, 0xffffff & x->x_gui.x_lcol);
     else sprintf(buf, "pdtk_iemgui_dialog %%s mknob \
-	    --------dimension(pix):-------- %d %d width: %d %d mouse: \
+	    --------dimension(pix):-------- %d %d size: %d %d mouse: \
 	    -----------output-range:----------- %g left: %g right: %g \
 	    %d lin log %d %d empty %d \
 	    %s %s \
 	    %s %d %d \
 	    %d %d \
 	   #%06x #%06x #%06x\n",
-            x->x_gui.x_w, MKNOB_MINSIZE, x->x_gui.x_h, -1,
+            x->x_gui.x_w/IEMGUI_ZOOM(x), MKNOB_MINSIZE, x->x_gui.x_h/IEMGUI_ZOOM(x), -1,
             x->x_min, x->x_max, 0.0,/*no_schedule*/
             x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, x->x_steady, -1,/*no multi, but iem-characteristic*/
             srl[0]->s_name, srl[1]->s_name,
@@ -465,11 +453,29 @@ static void mknob_properties(t_gobj *z, t_glist *owner)
     gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
 }
 
-static void mknob_set(t_mknob *x, t_floatarg f)    /* bugfix */
+    /* compute numeric value (fval) from pixel location (val) and range */
+static t_float mknob_getfval(t_mknob *x)
 {
+    t_float fval;
+    int zoom = x->x_gui.x_h > 0 ? IEMGUI_ZOOM(x) : 1;
+    int zoomval = (x->x_gui.x_fsf.x_finemoved) ?
+        x->x_val/zoom : (x->x_val / (100*zoom)) * 100;
+
+    if (x->x_lin0_log1)
+        fval = x->x_min * exp(x->x_k * (double)(zoomval) * 0.01);
+    else fval = (double)(zoomval) * 0.01 * x->x_k + x->x_min;
+    if ((fval < 1.0e-10) && (fval > -1.0e-10))
+        fval = 0.0;
+    return (fval);
+}
+
+static void mknob_set(t_mknob *x, t_floatarg f)
+{
+    int old = x->x_val;
     double g;
 
-    if(x->x_reverse)    /* bugfix */
+    x->x_fval = f;
+    if (x->x_min > x->x_max)
     {
         if(f > x->x_min)
             f = x->x_min;
@@ -487,21 +493,19 @@ static void mknob_set(t_mknob *x, t_floatarg f)    /* bugfix */
         g = log(f/x->x_min)/x->x_k;
     else
         g = (f - x->x_min) / x->x_k;
-    x->x_val = (int)(100.0*g + 0.49999);
+    x->x_val = (x->x_gui.x_h > 0 ? IEMGUI_ZOOM(x) : 1) * (int)(100.0*g + 0.49999);
     x->x_pos = x->x_val;
-    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
+    if(x->x_val != old)
+    	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
 }
 
 static void mknob_bang(t_mknob *x)
 {
     double out;
 
-    if(x->x_lin0_log1)
-        out = x->x_min*exp(x->x_k*(double)(x->x_val)*0.01);
-    else
-        out = (double)(x->x_val)*0.01*x->x_k + x->x_min;
-    if((out < 1.0e-10)&&(out > -1.0e-10))
-        out = 0.0;
+    if (pd_compatibilitylevel < 46)
+        out = mknob_getfval(x);
+    else out = x->x_fval;
     outlet_float(x->x_gui.x_obj.ob_outlet, out);
     if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
         pd_float(x->x_gui.x_snd->s_thing, out);
@@ -525,8 +529,7 @@ static void mknob_dialog(t_mknob *x, t_symbol *s, int argc, t_atom *argv)
     else
         x->x_steady = 0;
     sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
-    //x->x_gui.x_h = iemgui_clip_size(h);
-    //x->x_gui.x_w = iemgui_clip_size(w);
+
     mknob_check_wh(x, w, h);
     mknob_check_minmax(x, min, max);
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
@@ -561,6 +564,7 @@ static void mknob_motion(t_mknob *x, t_floatarg dx, t_floatarg dy)
         x->x_pos -= 50;
         x->x_pos -= x->x_pos%100;
     }
+    x->x_fval = mknob_getfval(x);
     if(old != x->x_val)
     {
         (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
@@ -585,6 +589,7 @@ static void mknob_motion_circular(t_mknob *x, t_floatarg dx, t_floatarg dy)
     else if(x->x_pos>(27000-100)) x->x_pos=(27000-100);
 
     x->x_val=x->x_pos;
+    x->x_fval = mknob_getfval(x);
 
     if(old != x->x_val)
     {
@@ -606,11 +611,10 @@ static void mknob_motion_fullcircular(t_mknob *x, t_floatarg dx, t_floatarg dy)
     alpha=atan2(xm-xc,ym-yc)*180.0/M_PI;
 
     x->x_pos=(int)(36000-alpha*100.0)%36000;
-    /*if(x->x_pos>31500) x->x_pos=0;
-    else if(x->x_pos>(27000-100)) x->x_pos=(27000-100);*/
 
     if(x->x_pos>(36000-100)) x->x_pos=(36000-100);
     x->x_val=x->x_pos;
+    x->x_fval = mknob_getfval(x);
 
     if(old != x->x_val)
     {
@@ -630,6 +634,7 @@ static void mknob_click(t_mknob *x, t_floatarg xpos, t_floatarg ypos,
         x->x_val = 100*x->x_H - 100;
     if(x->x_val < 0)
         x->x_val = 0;
+    x->x_fval = mknob_getfval(x);
     x->x_pos = x->x_val;
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
     mknob_bang(x);
@@ -651,6 +656,11 @@ static int mknob_newclick(t_gobj *z, struct _glist *glist,
 {
     t_mknob *x = (t_mknob *)z;
 
+	if(x->x_show_io && !x->x_gui.x_fsf.x_selected) {
+		x->x_show_io = 0;
+		mknob_draw_io(x, glist);
+	}
+    
     if(doit)
     {
         mknob_click( x, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift,
@@ -665,10 +675,10 @@ static int mknob_newclick(t_gobj *z, struct _glist *glist,
 
 static void mknob_size(t_mknob *x, t_symbol *s, int ac, t_atom *av)
 {
-    int w=(int)atom_getintarg(0, ac, av),
-        h=x->x_gui.x_h;
+    int w = (int)atom_getintarg(0, ac, av);
+    int h = x->x_gui.x_h;
 
-    if(ac > 1) h=(int)atom_getintarg(1, ac, av);
+    if(ac > 1) h= (int)atom_getintarg(1, ac, av);
 
     mknob_check_wh(x, w, h);
     iemgui_size((void *)x, &x->x_gui);
@@ -729,7 +739,7 @@ static void mknob_log(t_mknob *x)
 static void mknob_lin(t_mknob *x)
 {
     x->x_lin0_log1 = 0;
-    x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);
+    x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w/IEMGUI_ZOOM(x) - 1);
 }
 
 static void mknob_init(t_mknob *x, t_floatarg f)
@@ -747,46 +757,31 @@ static void mknob_float(t_mknob *x, t_floatarg f)
     double out;
 
     mknob_set(x, f);
-    if(x->x_lin0_log1)
-        out = x->x_min*exp(x->x_k*(double)(x->x_val)*0.01);
-    else
-        out = (double)(x->x_val)*0.01*x->x_k + x->x_min;
-    if((out < 1.0e-10)&&(out > -1.0e-10))
-        out = 0.0;
     if(x->x_gui.x_fsf.x_put_in2out)
+        mknob_bang(x);
+}
+
+static void mknob_zoom(t_mknob *x, t_floatarg f)
+{
+    if(x->x_gui.x_h > 0) 
     {
-        outlet_float(x->x_gui.x_obj.ob_outlet, out);
-        if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
-            pd_float(x->x_gui.x_snd->s_thing, out);
+    /* scale current pixel value */
+        x->x_val = (IEMGUI_ZOOM(x) == 2 ? (x->x_val)/2 : (x->x_val)*2);
+        x->x_H = (IEMGUI_ZOOM(x) == 2 ? (x->x_H)/2 : (x->x_H)*2);
+        x->x_pos = x->x_val;
     }
+    iemgui_zoom(&x->x_gui, f);
 }
 
 static void mknob_loadbang(t_mknob *x, t_floatarg action)
 {
-    /*if(!sys_noloadbang && x->x_gui.x_isa.x_loadinit)
+    if (action == LB_LOAD && x->x_gui.x_isa.x_loadinit)
     {
         (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
         mknob_bang(x);
-    }*/
-    if (action == LB_LOAD && x->x_gui.x_isa.x_loadinit)
-        mknob_bang(x);
+    }
 }
 
-/*static void mknob_list(t_mknob *x, t_symbol *s, int ac, t_atom *av)
-{
-    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);
-
-    if(l < 0)
-    {
-	if(IS_A_FLOAT(av,0))
-	    mknob_float(x, atom_getfloatarg(0, ac, av));
-    }
-    else if(l > 0)
-    {
-	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
-	canvas_fixlinesfor(x->x_gui.x_glist, (t_text*)x);
-    }
-}*/
 
 extern void iemgui_all_colfromload(t_iemgui *iemgui, int *bflcol);
 //extern void iemgui_all_loadcolors(t_iemgui *iemgui, t_atom*bcol, t_atom*fcol, t_atom*lcol);
@@ -857,6 +852,9 @@ static void *mknob_new(t_symbol *s, int argc, t_atom *argv)
 
         iemgui_all_colfromload(&x->x_gui, bflcol);
     } else {
+        iem_inttosymargs(&x->x_gui.x_isa, 0);
+        iem_inttofstyle(&x->x_gui.x_fsf, 0);
+ 
         x->x_gui.x_bcol = 0xFCFCFC;
         x->x_gui.x_fcol = 0x00;
         x->x_gui.x_lcol = 0x00;
@@ -935,16 +933,17 @@ static void *mknob_new(t_symbol *s, int argc, t_atom *argv)
 
     mknob_check_minmax(x, min, max);
 
-    x->x_thick = 0;
     iemgui_verify_snd_ne_rcv(&x->x_gui);
+    iemgui_newzoom(&x->x_gui);
+    x->x_fval = mknob_getfval(x);
+    x->x_show_io = 0;
+    x->x_io_visible = 0;
     outlet_new(&x->x_gui.x_obj, &s_float);
     return (x);
 }
 
 static void mknob_free(t_mknob *x)
 {
-    /*if(x->x_gui.x_fsf.x_selected)
-    pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);*/
     if(x->x_gui.x_fsf.x_rcv_able)
         pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
     gfxstub_deleteforkey(x);
@@ -957,20 +956,15 @@ void canvas_mknob(t_glist *gl, t_symbol *s, int argc, t_atom *argv)
     canvas_iemguis(gl, gensym("mknob"));
 }
 
-
-
 void mknob_setup(void)
 {
     check_compat();
 
     mknob_class = class_new(gensym("mknob"), (t_newmethod)mknob_new,
                             (t_method)mknob_free, sizeof(t_mknob), 0, A_GIMME, 0);
-#ifndef GGEE_mknob_COMPATIBLE
-//    class_addcreator((t_newmethod)mknob_new, gensym("mknob"), A_GIMME, 0);
-#endif
+
     class_addbang(mknob_class,mknob_bang);
     class_addfloat(mknob_class,mknob_float);
-    //class_addlist(mknob_class, mknob_list);
     class_addmethod(mknob_class, (t_method)mknob_click, gensym("click"),
                     A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
     class_addmethod(mknob_class, (t_method)mknob_motion, gensym("motion"),
@@ -992,8 +986,9 @@ void mknob_setup(void)
     class_addmethod(mknob_class, (t_method)mknob_lin, gensym("lin"), 0);
     class_addmethod(mknob_class, (t_method)mknob_init, gensym("init"), A_FLOAT, 0);
     class_addmethod(mknob_class, (t_method)mknob_steady, gensym("steady"), A_FLOAT, 0);
-    /*if(!iemgui_key_sym)
-    iemgui_key_sym = gensym("#keyname");*/
+    class_addmethod(mknob_class, (t_method)mknob_zoom,
+        gensym("zoom"), A_CANT, 0);
+
     mknob_widgetbehavior.w_getrectfn =    mknob_getrect;
     mknob_widgetbehavior.w_displacefn =   iemgui_displace;
     mknob_widgetbehavior.w_selectfn =     iemgui_select;
@@ -1001,12 +996,12 @@ void mknob_setup(void)
     mknob_widgetbehavior.w_deletefn =     iemgui_delete;
     mknob_widgetbehavior.w_visfn =        iemgui_vis;
     mknob_widgetbehavior.w_clickfn =      mknob_newclick;
-    //mknob_widgetbehavior.w_propertiesfn = mknob_properties;
-    //mknob_widgetbehavior.w_savefn =       mknob_save;
+
     class_setwidget(mknob_class, &mknob_widgetbehavior);
 
     class_setsavefn(mknob_class, mknob_save);
     class_setpropertiesfn(mknob_class, mknob_properties);
+
     class_addmethod(canvas_class, (t_method)canvas_mknob, gensym("mknob"),
                     A_GIMME, A_NULL);
 }
